//=======================================================================
// Author: Donovan Parks
//
// Copyright 2010 Donovan Parks
//
// This file is part of NaiveBayes.
//
// NaiveBayes is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// NaiveBayes is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with NaiveBayes.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#include "stdafx.h"

#include "FastaIO.hpp"
#include "KmerModel.hpp"
#include "TaxonomyIO.hpp"
#include "Utils.hpp"

int main(int argc, char* argv[])
{
	if(argc != 5)
	{
		std::cout << "nb-train v1.0 by Donovan Parks, Norm MacDonald, and Rob Beiko" << std::endl;
		std::cout << std::endl;
		std::cout << "Usage: nb-train <N> <taxonomy file> <input sequences file> <model output directory>" << std::endl;
		std::cout << std::endl;
		std::cout << "N: is the desired n-mer length." << std::endl;
		std::cout << std::endl;
		std::cout << "taxonomy file: indicates taxonomic information for each sequence that will be used during training." << std::endl; 
		std::cout << std::endl;
		std::cout << "input sequences file: text file indicating the multi-FASTA files to be models from." << std::endl;  
		std::cout << std::endl;
		std::cout << "model output directory: directory to write model." << std::endl;   
		std::cout << std::endl;
		std::cout << std::endl;
		std::cout << "Typical usage:" << std::endl;
		std::cout << "  nb-train 10 ./taxonomy.txt ./training/sequences.txt ./models/genomes/" << std::endl;
		std::cout << std::endl;

		return 0;
	}

	const uint KMER_SIZE = atoi(argv[1]);
	std::string taxonomyFile = argv[2];
	std::string inputSeqFile = argv[3];
	std::string outputDir = argv[4];

	// read file indicating taxonomic classification of each sequence
	std::cout << "Reading taxonomy file... " << std::endl;
	TaxonomyIO taxonomyIO;
	taxonomyIO.open(taxonomyFile);

	uint numStrains = taxonomyIO.numCategories(STRAIN_RANK);
	std::cout << "  Number of strains: " << numStrains << std::endl << std::endl;

	// train model for each species
	std::cout << "Training models..." << std::endl;
	uint numModels = 0;
	std::ifstream modelStream(inputSeqFile.c_str(), std::ios::in);
	while(!modelStream.eof())
	{
		std::string line;
		getline(modelStream, line);

		if(line.empty())
			continue;

		std::string modelName = line.substr(line.find_last_of('/')+1, std::string::npos);
		modelName = modelName.substr(0, modelName.find_last_of('.'));

		std::cout << "  Processing model " << modelName << std::endl;

		KmerModel kmerModel(KMER_SIZE);
		kmerModel.name(modelName);

		numModels++;		
	
		FastaIO fastaIO;
		
		bool bOK = fastaIO.open(line);
		if(!bOK)
		{
			std::cerr << "Error opening file: " << line << std::endl;
			return -1;
		}

		while(true)
		{
			SeqInfo seqInfo;
			bool bNextSeq = fastaIO.nextSeq(seqInfo);
			if(!bNextSeq)
				break;

			seqInfo.taxonomy = taxonomyIO.taxonomy(seqInfo.seqId);

			bool bOK = kmerModel.constructModel(seqInfo);
			if(!bOK)
			{
				std::cerr << "ERROR BUILDING MODEL!" << std::endl;
				return -1;
			}
		}	

		kmerModel.calculateConditionalProbabilities();
		kmerModel.write(outputDir + modelName + ".txt");
	}

	std::cout << std::endl << std::endl;
	std::cout << "Number of models: " << numModels << std::endl;

	return 0;
}

